@@ -1349,12 +1349,20 @@ cookies = CookieContainer()
 
 class NotABackgroundTaskState():
     def __setitem__(self, key, value):
-        raise Exception("Cannot access anvil.server.task_status outside a background task")
+        raise Exception("Cannot access anvil.server.task_state outside a background task")
 
     def __getitem__(self, item):
-        raise Exception("Cannot access anvil.server.task_status outside a background task")
+        raise Exception("Cannot access anvil.server.task_state outside a background task")
 
 
+# Raise event on all sessions in the current environment, or on a specific session
+def raise_event(name, payload=None, session_id=None):
+    anvil.server.call("anvil.private.raise_event", name, payload, session_id=session_id)
 
 
+def list_client_sessions():
+    return anvil.server.call("anvil.private.list_sessions")
 
+
+def get_client_session(session_id):
+    return anvil.server.call("anvil.private.get_session_data", session_id)
